package kz.arta.test.blocking.mysql;

import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import java.sql.Connection;

/**
 * Поток проверки соединений mysql arta-synergy
 *
 * @author MGetmanov
 * @since 28.05.15 8:43
 */
public class ArtaMysqlCheckConnThread extends Thread {
    private static final Logger LOGGER = LoggerFactory.getLogger(ArtaMysqlCheckConnThread.class);
    private StackTraceElement[] elements;

    private Connection con = null;
    private static boolean isEnabled = false;
    private static long timeout = 0;

    private static boolean closeConnections = false;

    static {
        if (ArtaMysqlConfigManager.getProps().containsKey(ArtaMysqlConfigManager.ConfigKeyProperties.observe_connections.name()) &&
                DataExtractorCommon.getBoolean(ArtaMysqlConfigManager.getProps().get(ArtaMysqlConfigManager.ConfigKeyProperties.observe_connections.name()))) {
            isEnabled = true;
            timeout = DataExtractorCommon.getLongStat(ArtaMysqlConfigManager.getProps().get(ArtaMysqlConfigManager.ConfigKeyProperties.observe_connection_timeout.name()));
            closeConnections = DataExtractorCommon.getBoolean(ArtaMysqlConfigManager.getProps().get(ArtaMysqlConfigManager.ConfigKeyProperties.close_observing_connection.name()));
            if (timeout < 180000) {
                timeout = 180000;
            }
        }
        LOGGER.info("Observing opened connections is " + (isEnabled ? "enabled" : "disabled"));
        if (isEnabled) {
            LOGGER.info("Started observing connections with parameters: timeout=" + timeout + "ms; close_connections=" + closeConnections);
            if (closeConnections) {
                LOGGER.error("WARNING! Closing connections in production mode is a VERY BAD IDEA. If you really need to do this " +
                        "contact developers.");
            }
        }
    }

    private ArtaMysqlCheckConnThread(StackTraceElement[] elements, Connection con) {
        this.elements = elements;
        this.con = con;
    }


    @Override
    public void run() {
        try {
            Thread.sleep(timeout);
        } catch (InterruptedException e) {
            LOGGER.error(e.getMessage(), e);
        }
        try {
            if (con != null && !con.isClosed()) {
                StringBuilder str = new StringBuilder();
                for (StackTraceElement element : elements) {
                    str.append("\n\t");
                    str.append(element.toString());
                }
                LOGGER.error("Connection was not closed within allowed timeout [" + timeout + "] !" + str.toString());
                if (closeConnections) {
                    LOGGER.error("Closing connection for you");
                    try {
                        con.close();
                    } catch (Exception exc) {
                        LOGGER.error(exc.getMessage(), exc);
                    }
                }
            }
        } catch (Exception exc) {
            LOGGER.error(exc.getMessage(), exc);
        }
    }

    public static void startObserveConnection(Connection con) {
        if (!isEnabled) {
            return;
        }
        new ArtaMysqlCheckConnThread(Thread.currentThread().getStackTrace(), con).start();
    }
}