package kz.arta.test.blocking.mysql;

import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import javax.naming.InitialContext;
import javax.sql.DataSource;
import java.sql.Connection;
import java.sql.SQLException;

/**
 * Класс предназначен для управления соединениями с базой данных
 * Для получения соединения необходимо использовать метод getXAConnection
 * Для закрытия соединения необходимо использовать метод  closeConnection
 * User: topa
 * Date: 15.07.2008
 * Time: 12:34:40
 */
public class ArtaMysqlConnectionPool {
    private static final Logger LOGGER = LoggerFactory.getLogger(ArtaMysqlConnectionPool.class);

    private static final String POOL_NAME = "java:/SynergyItsm";

    /**
     * Метод для получения соединения из пула
     *
     * @return - открытое соединение
     */
    public static Connection getXAConnection() {
        Connection con = null;
        try {
            InitialContext ctx = new InitialContext();
            DataSource ds = (DataSource) ctx.lookup(POOL_NAME);
            con = ds.getConnection();
            ArtaMysqlCheckConnThread.startObserveConnection(con);
            return new ConnectionImpl(con);
        } catch (Exception e) {
            LOGGER.error(e.getMessage(), e);
            closeConnection(con);
            throw new IllegalStateException(e);
        }
    }

    /**
     * Метод для закрытия соединения
     *
     * @param con - соединение которое необходимо закрыть
     */
    public static void closeConnection(Connection con) {
        if (con != null) {
            try {
                con.close();
            } catch (SQLException e) {
                LOGGER.error(e.getMessage(), e);
            }
        }
    }
}
