package kz.arta.test.blocking.mysql;

import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import java.io.File;
import java.io.FileInputStream;
import java.net.MalformedURLException;
import java.net.URL;
import java.util.Collection;
import java.util.HashSet;
import java.util.Properties;
import java.util.regex.Matcher;
import java.util.regex.Pattern;

public class DbConfig {
    private static Logger LOGGER = LoggerFactory.getLogger(ArtaMysqlConfigManager.class);
    private static final String DEFAULT_EAR_NAME = "Synergy";

    private String ear_name = "Synergy";

    private Properties props =  new Properties();

    public DbConfig() {
    }

    public Properties getProps() {
        return props;
    }

    public void initialize() {
        String cfgFolder = getCfgURL();

        LOGGER.info("JBoss cfg directory is " + cfgFolder + ". Trying read settings from it");

        try {

            File file = new File(getConfFilePath("arta/management/db.properties"));
            if (file.exists()) {
                FileInputStream fis = new FileInputStream(file);
                props.load(fis);
                fis.close();
            } else {
                throw new RuntimeException("File " + file.getAbsolutePath() + " does not exist!");
            }
        } catch (Exception exc) {
            LOGGER.error(exc.getMessage(), exc);
            LOGGER.error("Failed to read settings from " + cfgFolder);
        }

        try {
            if (ear_name != null && ear_name.length() > 0) {
                LOGGER.debug("Going to read jcr mapped resource name");
            }

        } catch (Exception exc) {
            LOGGER.error(exc.getMessage(), exc);
        }
    }

    /**
     * Инициализация кэша очередей для документов
     */
    private void initDocExents() {
        final String docQueue = "java:jboss/exported/jms/queue/DocIndex";
        final Collection<String> queue = new HashSet<String>();
        queue.add(docQueue);
    }

    public String getEarName() {
        String classLoader = Thread.currentThread().getContextClassLoader().toString();
        Pattern p = Pattern.compile("(?:deployment.)([A-Za-z0-9_-[.]]+)(.ear)");
        Matcher m = p.matcher(classLoader);
        if (m.find()) {
            return m.group(1);
        } else {
            return "";
        }
    }

    public String getCfgURL() {
        //For JBoss AS5
        String cfgDir = System.getProperty("jboss.server.config.url");
        //For JBoss AS7
        if (cfgDir == null) {
            cfgDir = System.getProperty("jboss.server.config.dir");
            if (cfgDir != null) {
                cfgDir = "file://" + cfgDir;
            } else {
                //If nothing helps
                cfgDir = "file:///home/topa/services/jboss/server/default/conf";
            }
        }
        return cfgDir;
    }


    public String getCfgDirectory() {
        //For JBoss AS5
        String cfgDir = System.getProperty("jboss.server.config.url");
        if (cfgDir != null && cfgDir.startsWith("file://")) {
            try {
                URL url = new URL(cfgDir);
                return url.getFile();
            } catch (MalformedURLException e) {
                LOGGER.error(e.getMessage(), e);
            }
        }
        //For JBoss AS7
        if (cfgDir == null) {
            cfgDir = System.getProperty("jboss.server.config.dir");
            if (cfgDir != null) {
                return cfgDir;
            } else {
                //If nothing helps
                return "/home/topa/services/jboss/server/default/conf";
            }
        }
        return cfgDir;
    }

    public String getSynergyDS() {
        if (ear_name.equals(DEFAULT_EAR_NAME)) {
            return "java:/SynergyDS";
        }
        return "java:jboss/datasources/" + ear_name + "/synergy";
    }

    public String getConfFilePath(String relativePath) {
        if (ear_name == null || ear_name.equals(DEFAULT_EAR_NAME))
            return getCfgDirectory() + "/" + relativePath;
        return getCfgDirectory() + "/" + ear_name + "/" + relativePath;
    }

}
